@@ -32,8 +32,9 @@ from typing import Union
 from requests.exceptions import HTTPError
 
 from internetarchive.cli.cli_utils import (
+    MetadataAction,
+    QueryStringAction,
     get_args_dict,
-    prepare_args_dict,
     validate_identifier,
 )
 from internetarchive.utils import (
@@ -78,8 +79,10 @@ def setup(subparsers):
                         help=("When uploading data from stdin, "
                              "this option sets the remote filename"))
     parser.add_argument("-m", "--metadata",
+                        nargs="+",
+                        action=MetadataAction,
                         metavar="KEY:VALUE",
-                        action="append",
+                        default={},
                         help="Metadata to add to your item")
     parser.add_argument("--spreadsheet",
                         type=argparse.FileType("r", encoding="utf-8-sig"),
@@ -88,7 +91,9 @@ def setup(subparsers):
                         type=argparse.FileType("r"),
                         help="Upload files with file-level metadata via a file_md.jsonl file")
     parser.add_argument("-H", "--header",
-                        action="append",
+                        nargs="+",
+                        action=QueryStringAction,
+                        default={},
                         help="S3 HTTP headers to send with your request")
     parser.add_argument("-c", "--checksum",
                         action="store_true",
@@ -122,9 +127,6 @@ def setup(subparsers):
     parser.add_argument("--keep-directories",
                         action="store_true",
                         help="Keep directories in the supplied file paths for the remote filename")
-    parser.add_argument("--no-scanner",
-                        action="store_true",
-                        help="Do not set the scanner field in meta.xml")
     parser.add_argument("--status-check",
                         action="store_true",
                         help="Check if S3 is accepting requests to the given item")
@@ -197,17 +199,12 @@ def main(args, parser): # noqa: PLR0912,C901
     """
     Main entry point for 'ia upload'.
     """
-
     check_if_file_arg_required(args, parser)
 
     if uploading_from_stdin(args) and not args.remote_name:
         parser.error("When uploading from stdin, "
                      "you must specify a remote filename with --remote-name")
 
-    # Prepare args key:val dicts
-    args.metadata = prepare_args_dict(args.metadata, parser, arg_type="metadata")
-    args.header = prepare_args_dict(args.header, parser, arg_type="header")
-
     if args.status_check:  # TODO: support for checking if a specific bucket is overloaded
         if args.session.s3_is_overloaded():
             print(f"warning: {args.identifier} is over limit, and not accepting requests. "
@@ -229,10 +226,6 @@ def main(args, parser): # noqa: PLR0912,C901
         verbose = False
     else:
         verbose = True
-    if args.no_scanner:
-        set_scanner = False
-    else:
-        set_scanner = True
     if args.size_hint:
         args.header["x-archive-size-hint"] = args.size_hint
     if not args.header.get("x-archive-keep-old-version") \
@@ -241,11 +234,11 @@ def main(args, parser): # noqa: PLR0912,C901
 
     if args.file_metadata:
         try:
-            with open(args.file_metadata) as fh:
-                args.file_metadata = json.load(fh)
+            with open(args.file_metadata.name) as fh:
+                args.file = json.load(fh)
         except JSONDecodeError:
             args.file = []
-            with open(args.file_metadata) as fh:
+            with open(args.file_metadata.name) as fh:
                 for line in fh:
                     j = json.loads(line.strip())
                     args.file.append(j)
@@ -255,7 +248,6 @@ def main(args, parser): # noqa: PLR0912,C901
         "headers": args.header,
         "debug": args.debug,
         "queue_derive": queue_derive,
-        "set_scanner": set_scanner,
         "verbose": verbose,
         "verify": args.verify,
         "checksum": args.checksum,
@@ -308,7 +300,13 @@ def main(args, parser): # noqa: PLR0912,C901
         for _r in _upload_files(item, files, upload_kwargs):
             if args.debug:
                 break
-            if (not _r.status_code) or (not _r.ok):
+
+            # Check if Response is empty first (i.e. --checksum)
+            # TODO: Should upload return something other than an empty Response
+            # object if checksum is set and the file is already in the item?
+            if _r.status_code is None:
+                pass
+            elif not _r.ok:
                 errors = True
             else:
                 if args.open_after_upload:
